/* ------------------------------------------------------------------------ 

   ExploristConvert.java 

   Projekt: Webfox 

  --------------------------------------------------------------------------*/

/* ---
created: 04.07.2008 a.sander

$History:$
--- */

package de.picosaan.explorist;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Iterator;

import org.jdom.Content;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;


/**
 * Application that gets name/path of a GPX file as parameter. Makes sure that no name
 * is longer that 20 characters and that there are no names that are equal in the first
 * 20 characters. Also adds the first character of the container name to the owner name
 *
 **/
public class ExploristConvert
  {

  /**
   * Main
   */
  public static void main(String[] args)
    {
    /* --- Check that we have exactly one parameter --- */
    if (args.length == 2)
      ProcessFile(args[0], args[1]);
    else
      System.out.println("Aufruf: ExploristConvert <input-gpx-datei> <output-gpx-datei>");
    }

  /**
   * Loads the file with the given name, makes the modifications and writes it back.
   * @param input_filename name/path of GPX file
   * @param output_filename name/path of the destination file
   */
  private static void ProcessFile(String input_filename, String output_filename)
    {
    Document doc;
    SAXBuilder builder;
    FileInputStream fi;
    FileOutputStream fo;
    
    try
      {
      /* --- Read the XML file from disk --- */
      builder = new SAXBuilder();
      fi = new FileInputStream(input_filename);
      doc = builder.build(fi);
      fi.close();
      
      /* --- Do the conversions --- */
      ProcessDocument(doc);
      
      /* --- Write the result to the output file --- */
      XMLOutputter fmt = new XMLOutputter();
      fmt.setFormat(Format.getPrettyFormat());
      
      fo = new FileOutputStream(output_filename);
      fmt.output(doc, fo);
      fo.close();
      
      /* --- Print the finished message --- */
      System.out.println("Fertig");
      }
    catch(Exception e)
      {
      System.out.println("Operation fehlgeschlagen: " + e.getMessage());
      }
    }
  
  /**
   * Makes the modifications to the XML document
   * @param doc XML document
   */
  private static void ProcessDocument(Document doc)
    {
    NameNormalizer normalizer;
    
    /* --- Read all names from the GPX file and build a normalizer from it --- */
    normalizer = BuildNormalizer(doc);
    
    /* --- Normalize the names --- */
    normalizer.normalize(20);

    /* --- Modify name and owner in the waypoints --- */
    DoModifications(doc, normalizer);
    }
  
  
  /**
   * Returns the GPX node from a given document
   * @param doc document
   * @return gpx node of the document
   */
  private static Element getGPX(Document doc)
    {
    Content content;
    
    /* --- Check the first element is a "gpx" element --- */
    content = doc.getContent(0);
    if (!(content instanceof Element) || ((Element) content).getName().compareTo("gpx") != 0)
      throw new IllegalArgumentException("Datei ist keine GPX Datei");

    return (Element) content;
    }
  
  /**
   * Load all waypoints from the doc and create a NameNormalizer from the data.
   * The waypoint name is used as key and the cache name is used as name 
   * @param doc document
   * @return filled name normalizer
   */
  @SuppressWarnings("unchecked")
  private static NameNormalizer BuildNormalizer(Document doc)
    {
    NameNormalizer normalizer;
    Element gpx;
    Namespace std;
    Namespace groundspeak;
    Iterator it;
    Element wpt;
    String key;
    String name;
    Element cache;

    /* --- Get the GPX node from the document --- */
    gpx = getGPX(doc);
    
    /* --- Create normalizer --- */
    normalizer = new NameNormalizer();
    
    /* --- get the standard namespace and the groundspeak namespace --- */
    std = gpx.getNamespace();
    groundspeak = Namespace.getNamespace("http://www.groundspeak.com/cache/1/0");
    
    /* --- Iterate over all waypoints --- */
    it = gpx.getChildren("wpt", std).iterator();
    while(it.hasNext())
      {
      /* --- Get the waypoint --- */
      wpt = (Element) it.next();
      
      /* --- Get the name of the waypoint --- */
      key = wpt.getChildText("name", std);
      
      /* --- Get the cache subelement --- */
      cache = wpt.getChild("cache", groundspeak);
      if (cache == null)
        continue;
      
      /* --- Get the name of the cache --- */
      name = cache.getChildText("name", groundspeak);
      
      /* --- Add the data to the normalizer --- */
      if (key != null && cache != null)
        normalizer.addName(key, name);
      }
    
    return normalizer;
    }
  

  /**
   * Modify cache name and owner name in the waypoints of the document
   * @param doc document to modify
   * @param normalizer normalizer with modified cache names per waypoint name
   */
  @SuppressWarnings("unchecked")
  private static void DoModifications(Document doc, NameNormalizer normalizer)
    {
    Element gpx;
    Namespace std;
    Namespace groundspeak;
    Iterator it;
    Element wpt;
    String key;
    String name;
    Element cache;
    Element name_element;
    String container_size;
    Element owner_element;

    /* --- Get the GPX node from the document --- */
    gpx = getGPX(doc);
    
    /* --- get the standard namespace and the groundspeak namespace --- */
    std = gpx.getNamespace();
    groundspeak = Namespace.getNamespace("http://www.groundspeak.com/cache/1/0");
    
    /* --- Iterate over all waypoints --- */
    it = gpx.getChildren("wpt", std).iterator();
    while(it.hasNext())
      {
      /* --- Get the waypoint --- */
      wpt = (Element) it.next();
      
      /* --- Get the name of the waypoint --- */
      key = wpt.getChildText("name", std);
      
      /* --- Get the cache subelement --- */
      cache = wpt.getChild("cache", groundspeak);
      if (cache == null)
        continue;
      
      /* --- Check if there is a name replacement for the waypoint --- */
      name = normalizer.getName(key);
      if (name != null)
        {
        name_element = cache.getChild("name", groundspeak);
        if (name_element != null)
          name_element.setText(name);
        }
      
      /* --- Get the container size --- */
      container_size = cache.getChildText("container", groundspeak);
      
      /* --- Get the owner element --- */
      owner_element = cache.getChild("owner", groundspeak);
      
      /* --- Add this first char of the container to the beginning of the owner --- */
      if (container_size != null && container_size.length() > 0 && owner_element != null)
        owner_element.setText(container_size.substring(0, 1) + " " + owner_element.getText());
      }
    }
  }
