/* ------------------------------------------------------------------------ 

   NameNormalizer.java 

   Projekt: Webfox 

  --------------------------------------------------------------------------*/

/* ---
created: 03.07.2008 a.sander

$History:$
--- */

package de.picosaan.explorist;

import java.util.Arrays;
import java.util.Hashtable;
import java.util.Iterator;


/**
 * Store for key-name pairs. The Normalizer cuts the names to the first "n" characters
 * and makes sure that there are no name-doubles by adding "01", "02", ... to the end of
 * names where doubles would be created otherwise.<BR>
 * The names are changed that way, that the sort order is not changed.
 *
 **/
public class NameNormalizer
  {
  private class Data implements Comparable<Data>
    {
    String key;
    String name;
    
    public int compareTo(Data o)
      {
      return name.compareTo(o.name);
      }
    }
  
  /* ----------------------------------------------------------- */
  /* ----------------------------------------------------------- */
  /* ----------------------------------------------------------- */
  /* ----------------------------------------------------------- */
  /* ----------------------------------------------------------- */
  /* ----------------------------------------------------------- */
  
  /* --- Storage for key-name pairs --- */
  private Hashtable<String, String> storage;
  
  /**
   * Constructor
   */
  public NameNormalizer()
    {
    storage = new Hashtable<String, String>();
    }
  
  /**
   * Add a key/name pair to the storage.<P>
   * 
   * Entries with the same key are overwritten
   * @param key key value
   * @param name name value
   */
  public void addName(String key, String name)
    {
    storage.put(key, name);
    }
  
  /**
   * Return the name for a given key. If the normalize() function is not called yet,
   * then the original name is returned, otherwise the normalized one
   * @param key key as used in addName()
   * @return name that is assigned to the key or null if key is unknown
   */
  public String getName(String key)
    {
    return storage.get(key);
    }
  
  /**
   * Normalizes the names set in addName(). The original name is no more available
   * after the normalizing. Normalizing is done as described in the class description
   * 
   * @param length maximum length of the normalized names
   */
  public void normalize(int length)
    {
    Data[] data;
    Iterator<String> it;
    int off = 0;
    String current_name = "";
    int current_count = 0;
    
    /* --- Build an array with all data --- */
    data = new Data[storage.size()];
    it = storage.keySet().iterator();
    for (off = 0; off<data.length; off++)
      {
      data[off] = new Data();
      data[off].key = it.next();
      data[off].name = storage.get(data[off].key);
      }
    
    /* --- Sort the array by name [name is the natural sort order of "Data"] --- */
    Arrays.sort(data);
    
    /* --- Clip all names by the given length --- */
    for (off = 0; off<data.length; off++)
      {
      if (data[off].name.length() > length)
        data[off].name = data[off].name.substring(0, length);
      }
    
    /* --- Check for conflicts and remove them --- */
    off = 0;
    while (off<data.length-1)
      {
      /* --- remember the name we are currently processing --- */
      current_name = data[off].name;
      current_count = 1;
      
      /* --- If the name of the next element is the same as the name we are currently
       *     processing, then there is a conflict and we have to rename this element
       *     and all following elements with this name
       */
      if (data[off+1].name.compareTo(current_name) == 0)
        {
        while(data[off].name.compareTo(current_name) == 0)
          {
          data[off].name = createNumberName(current_name, current_count, length);
          current_count++;
          off++;
          }
        }
      /* --- If the names do not match, just proceed with the next name --- */
      else
        off++;
      }
    
    /* --- Modify the names in the Hashtable --- */
    for (off = 0; off<data.length; off++)
      {
      storage.put(data[off].key, data[off].name);
      }
    }
  
  /**
   * Adds a number with at least two digits to "source" if the combination of
   * source and number is longer than max_length then source is shorten accordingly.
   * At least one character from the source is used even if source and number are longer 
   * than max_length
   * @param source text string the number has to be added to
   * @param number number to be added
   * @param max_length maximum length of source + number
   * @return source (eventually shorten) + number
   */
  private String createNumberName(String source, int number, int max_length)
    {
    String result = "";
    String number_str;
    int source_length;
    
    /* --- Build a string from the number --- */
    number_str = String.format("%02d", number);
    
    /* --- Calculate the allowed length of source --- */
    source_length = max_length - number_str.length();
    if (source_length < 1)
      source_length = 1;
    
    /* --- Shorten source if necessary and add the number --- */
    if (source.length() > source_length)
      result = source.substring(0, source_length) + number_str;
    else
      result = source + number_str;
    
    return result;
    }
  }
